pub struct MapLinearWithDim<const N: usize = 3> { /* private fields */ }Expand description
Linear mapping for 1D array. N is the number of thread dimensions. width is the chunking window. The array is divided into chunks along threads until all elements are covered.
Examples
Given: bdim = (x=4, y=1, z=1) gdim = (x=1, y=1, z=1) arr: [T; 8] width = 2
With width = 2, the mapping assign two continuous elements to thread 0-3:
[0, 0, 1, 1, 2, 2, 3, 3]
width = 1, the mapping assign two non-continuous elements to thread 0-3: [0, 1, 2, 3, 0, 1, 2, 3]
TODO: deprecate MapLinear and MapLinearWithDim in favor of reshape_map! macro.
Implementations