pub trait DynamicSharedAllocBuilder {
// Required method
fn smem_alloc(&self) -> DynamicSharedAlloc;
}Expand description
This trait is implemented for kernel config struct to provide dynamic shared memory allocation.
pub trait DynamicSharedAllocBuilder {
// Required method
fn smem_alloc(&self) -> DynamicSharedAlloc;
}This trait is implemented for kernel config struct to provide dynamic shared memory allocation.