pub struct Map2D {
pub x_size: usize,
}Expand description
This mapping strategy is useful when we want to reshape a 1D array into a 2D array and then distribute one element to a thread one by one until consuming all. It creates a new non-continuous partition for each thread.
- IndexType is (usize, usize)
Example: Example:
- array: [T; 20]
- x_size = 5 => y_size = 4
- dim: x=2, y=2, z=1
0 1 2 3 4 5
┌───┬───┬───┬───┬───┐
│0,0│1,0│0,0│1,0│0,0│
├───┼───┼───┼───┼───┤
│0,1│1,1│0,1│1,1│0,1│
├───┼───┼───┼───┼───┤
│0,0│1,0│0,0│1,0│0,0│
├───┼───┼───┼───┼───┤
│0,1│1,1│0,1│1,1│0,1│
└───┴───┴───┴───┴───┘In this case,
- thread(1,0) and (1,1) should only have access to a shape of 2*2 = 4 elements.
- thread (0,0) and (0,1) have access to a shape of 3*2 = 6 elements.
Fields
x_size: usizeImplementations